#include <stdlib.h>
#include <stdio.h>
#include <string.h>
/*#include <sys/types.h>*/
/*#include <sys/stat.h>*/
/*#include <libgnome/gnome-url.h>*/
/*#include <fcntl.h>*/

#include "search.h"
#include "stringhash.h"
#include "stringtokenizer.h"
#include "urlencoder.h"
#include "properties.h"

GHashTable *engine_table;
GList *englist;
GList *sublist;

/*  init_search_config() must be called before
	all other methods defined by this file... */
void init_search_config()
{
	englist = NULL;
	engine_table = g_hash_table_new ( (GHashFunc)string_hashcode,
								  (GCompareFunc)string_compare );
}


/*  Add a search engine from a ".netferretrc" file... */
void add_engine_from_config_line (gchar *line, gint lineno)
{
	EnginePtr eng;
	gchar* tmp;
	gchar* tmpcopy;
	gchar* idx;
	TokenizerPtr tok = new_tokenizer(line, "\t");
	
	/* allocate space for this engine */
#ifdef HDEBUG
	eng = h_alloc (sizeof (Engine), "eng", "add_engine_from_config_line", 39);
#else
	eng = g_malloc (sizeof (Engine));
#endif
	//printf ("allocated engine pointer\n");

	/* break up line, using "\t" as a tokenizer */
	tmp = next_token (tok); //strtok (line, "\t");
	//printf ("tmp: \"%s\"...\n", tmp);

#ifdef HDEBUG
	eng->list_name = h_alloc (strlen(tmp)+1, "eng->list_name", "add_engine_from_config_line", 46);
#else
	eng->list_name = g_malloc (strlen(tmp)+1);
#endif
	//printf ("allocated eng->list_name\n");

	strcpy (eng->list_name, tmp);
	//printf ("parsing engine: \"%s\"...\n", eng->list_name);
	
#ifdef HDEBUG
	h_free (tmp, "tmp", "add_engine_from_config_line", 52);
#else
	g_free (tmp);
#endif
	//printf ("g_free (tmp)\n");
	
	tmp = next_token (tok);
	//printf ("tmp: \"%s\"...\n", tmp);
	
	if ((strcmp (tmp, ON_SWITCH) == 0) || (strcmp (tmp, TRUE_SWITCH) == 0))
		eng->isvisible = TRUE;
	else
		eng->isvisible = FALSE;
	
#ifdef HDEBUG
	h_free (tmp, "tmp", "add_engine_from_config_line", 63);
#else
	g_free (tmp);
#endif
	//printf ("g_free (tmp)\n");

	//printf ("parsing engine: \"%s\"...\n", eng->list_name);

	/* break up the magic string into two strings...
	   the string before the $MAGIC identifier = magic1,
	   and the string after $MAGIC = magic2.
	*/
	tmp = next_token (tok);
	//printf ("tmp: \"%s\"...\n", tmp);
	//printf ("magic URL: \"%s\"\n", tmp);
	
#ifdef HDEBUG
	tmpcopy = h_alloc (strlen(tmp)+1, "tmpcopy", "add_engine_from_config_line", 76);
#else
	tmpcopy = g_malloc (strlen(tmp)+1);
#endif
	strcpy (tmpcopy, tmp);
	//printf ("tmpcopy: \"%s\"...\n", tmp);

	idx = strstr (tmpcopy, MAGIC);
	*idx = 0;
	//printf ("tmpcopy: \"%s\"...\n", tmp);
	

#ifdef HDEBUG
	eng->magic1 = h_alloc (strlen(tmpcopy)+1, "eng->magic1", "add_engine_from_config_line", 85);
#else
	eng->magic1 = g_malloc (strlen(tmpcopy)+1);
#endif

	strcpy (eng->magic1, tmpcopy);
	//printf ("eng->magic1: \"%s\"\n", eng->magic1);

	idx += strlen (MAGIC);

#ifdef HDEBUG
	eng->magic2 = h_alloc (strlen(idx)+1, "eng->magic2", "add_engine_from_config_line", 91);
#else
	eng->magic2 = g_malloc (strlen(idx)+1);
#endif
	strcpy (eng->magic2, idx);
	//printf ("eng->magic2: \"%s\"\n", eng->magic2);
	
	englist = g_list_append (englist, eng);
	//printf ("appended englist\n");
	
	g_hash_table_insert (engine_table, eng->list_name, eng);
	//printf ("appended engine_table\n");
		
	//printf ("\t... done making engine entries.\n");
#ifdef HDEBUG
	h_free (tmp, "tmp", "add_engine_from_config_line", 102);
	h_free (tmpcopy, "tmpcopy", "add_engine_from_config_line", 103);
#else
	g_free (tmp);
	g_free (tmpcopy);
#endif

	free_tokenizer(tok);
	//printf ("finished parsing config line.\n");	
}

void _make_default_engine_subset (gpointer data, gpointer user_data)
{
	EnginePtr eng = (EnginePtr)data;
	if (eng->isvisible)
		sublist = g_list_append (sublist, eng->list_name);
}

GList *get_default_engine_list()
{
	if (sublist) g_list_free (sublist);
	sublist = NULL;
	g_list_foreach (englist, (GFunc)_make_default_engine_subset, NULL);
	return sublist;
}

GList *get_engines_list()
{
	return englist;
}


EnginePtr engine_for_list_name (gchar* name)
{
	return (EnginePtr)g_hash_table_lookup (engine_table, name);
}

GHashTable *get_engine_table()
{
	return engine_table;
}


void url_show_with_options (gchar *url)
{
	gchar *browser = get_property ("browser");
	gint blen = strlen (browser);
	gint ulen = strlen (url);
	gint len = blen + ulen;
	gchar *exec;
	
#ifdef HDEBUG
	exec = h_alloc (len + 4, "exec", "url_show_with_options", 147);
#else
	exec = g_malloc (len + 4);
#endif
	strncpy (exec, browser, blen);
	*(exec+blen) = ' ';
	*(exec+blen+1) = '\"';
	strncpy (exec+blen+2, url, ulen);
	*(exec+len+2) = '\"';
	*(exec+len+3) = 0;
	//printf ("exec: %s\n", exec);
	
	gnome_execute_shell(NULL, exec);
	
#ifdef HDEBUG
	h_free(exec, "exec", "url_show_with_options", 158);
#else
	g_free(exec);
#endif
	
	//gnome_url_show(exec);
}


void do_search (QueryPtr query)
{
	gchar *url;
	gchar *copy;
	gint qlen, m1len, m2len;	
	gchar *qtext = escape (query->query);

	qlen = strlen (qtext);
	m1len = strlen (query->engine->magic1);
	m2len = strlen (query->engine->magic2);

#ifdef HDEBUG
	url = h_alloc (qlen + m1len + m2len + 1, "url", "do_search", 175);
#else
	url = g_malloc (qlen + m1len + m2len + 1);
#endif
	copy = url;
	
	//printf ("query->engine->magic1 \"%s\"\n", query->engine->magic1);
	//printf ("qtext \"%s\"\n", qtext);
	//printf ("query->engine->magic2 \"%s\"\n", query->engine->magic2);

	strncpy (copy, query->engine->magic1, m1len);
	copy += m1len;
	strncpy (copy, qtext, qlen);
	copy += qlen;
	strncpy (copy, query->engine->magic2, m2len);
	copy += m2len;
	*copy = 0;
	
	//printf ("show URL: \"%s\"\n", url);
	
	url_show_with_options(url);
		

#ifdef HDEBUG
	h_free (qtext, "qtext", "do_search", 194);
	h_free (url, "url", "do_search", 195);
#else
	g_free (qtext);
	g_free (url);
#endif
}



void dispose_engine (Engine *eng)
{
	//printf ("freeing engine: \"%s\"\n", eng->list_name);
#ifdef HDEBUG
	h_free (eng->list_name, "eng->list_name", "dispose_engine", 203);
	h_free (eng->magic1, "eng->magic1", "dispose_engine", 204);
	h_free (eng->magic2, "eng->magic2", "dispose_engine", 205);
	h_free (eng, "eng", "dispose_engine", 206);
#else
	g_free (eng->list_name);
	g_free (eng->magic1);
	g_free (eng->magic2);
	g_free (eng);
#endif
}


void dispose_engine_cb (gpointer key,
						gpointer value,
						gpointer user_data)
{
	dispose_engine ((Engine*)value);
}


void dispose_search_config()
{
	/* dispose each entry */
	g_hash_table_foreach (engine_table,
		(GHFunc)dispose_engine_cb, NULL);

	/* then dispose the engine_table */
	g_hash_table_destroy(engine_table);
}


