#include <memory.h>

#include "stringtokenizer.h"



TokenizerPtr new_tokenizer (gchar* text, gchar* delim)
{
	TokenizerPtr tok;
	
#ifdef HDEBUG
	tok = h_alloc (sizeof (Tokenizer), "tok", "new_tokenizer", 9);
#else
	tok = g_malloc (sizeof (Tokenizer));
#endif

	//printf ("new_tokenizer: \"%s\", \"%s\"\n", text, delim);
	tok->text = text;
	tok->delim = delim;
	tok->delim_len = strlen (delim);
	tok->offset = 0;
	return tok;
}


gint has_more_tokens(TokenizerPtr tok)
{
	//printf ("has_more_tokens testing: \"%s\"\n", (tok->text)+tok->offset);
	if (tok->offset >= strlen(tok->text)) return FALSE;
	return (strcspn ((tok->text)+tok->offset, tok->delim) != 0);
}


gint count_tokens (TokenizerPtr tok)
{
	gint i = 0;
	size_t this_offs = strcspn (tok->text+tok->offset, tok->delim);
	size_t offs = 0;
	gint len = strlen (tok->text);

	if (this_offs != 0)
	{
		offs+=this_offs;
		i++;
	}

	//printf ("count_tokens: %d, %d\n", this_offs, i);

	while ((this_offs != 0) && (offs < len))
	{
		this_offs = strcspn (tok->text+offs, tok->delim);
		i++;
	
	/*	printf ("count_tokens in \"%s\" | offset: %d, tokens: %d\n",
				tok->text+offs, this_offs, i);
	*/
		offs+=this_offs;
	}
	return i;
}


gint is_delim (TokenizerPtr tok, size_t pos)
{
	int i;
	for (i=0; i<tok->delim_len; i++)
	{
		if (tok->delim[i] == tok->text[pos])
			return TRUE;
	}
	return FALSE;
}


void remove_leading_delimiters (TokenizerPtr tok) {
	while (is_delim(tok, tok->offset) &&
		(tok->offset < strlen(tok->text)))
			tok->offset++;
}


/*	Ownership of return value is
	transferred to calling function. 
*/
gchar *next_token (TokenizerPtr tok)
{
	gchar *ret;
	gchar *ptroffset = (tok->text)+tok->offset;
	size_t len = strcspn (ptroffset, tok->delim);
	
#ifdef HDEBUG
	ret = h_alloc (len+1, "ret", "next_token", 83);
#else
	ret = g_malloc (len+1);
#endif
	memcpy (ret, ptroffset, len);
	ret[len] = 0;
	tok->offset += len;
	remove_leading_delimiters(tok);
	//printf ("next_token returning: \"%s\"\n", ret);
	return ret;
}


/*	Ownership of return value is
	transferred to calling function. 
*/
gchar *remaining_text (TokenizerPtr tok)
{
	gchar *ret;
	gchar *ptroffset = (tok->text)+tok->offset;
	size_t len = strlen (ptroffset)+1;
#ifdef HDEBUG
	ret = h_alloc (len, "ret", "remaining_text", 101);
#else
	ret = g_malloc (len);
#endif
	memcpy (ret, ptroffset, len);
	//printf ("remaining_text returning: \"%s\"\n", ret);
	return ret;
}


void free_tokenizer(TokenizerPtr tok)
{
#ifdef HDEBUG
	h_free (tok, "tok", "free_tokenizer", 110);
#else
	g_free (tok);
#endif
}
